// EQTxPacket.cpp

#include "EQServerSocket_stdafx.h"
#include "EQTxPacket.h"
#include "UDPServerSocket.h"

// CDatagramPacket constructor only sets the max packet size which is fine
// because it'll allocate the buffer and then the actual packet size is set\
// in init()
CEQTxPacket::CEQTxPacket(
		unsigned short header,
		unsigned short packetID,
		unsigned short ackID,
		const void* pData,
		size_t dataSize)
	: CDatagramPacket(700, dataSize + (header & EQ_MSG_ACK ? 10 : 8))
{
	getData().allocBuffer(this->m_szMaxPacketSize);
	void* pBuffer = getData().getData();
	int o = 0;

	CEQPacket::FillWord(pBuffer, o++ * 2, header);
	CEQPacket::FillWord(pBuffer, o++ * 2, packetID);

	if(header & EQ_MSG_ACK)
		CEQPacket::FillWord(pBuffer, o++ * 2, ackID);

	init(o * 2,	pData, dataSize);
}
/*
CEQTxPacket::CEQTxPacket(
		unsigned short header,
		unsigned short packetID,
		unsigned short ackID,
		unsigned short sequenceID,
		unsigned char messageID,
		const void* pData,
		size_t dataSize);
	: CDatagramPacket(dataSize + 10 (header & EQ_MSG_ACK ? 2 : 0) + (header & EQ_MSG_MSG ? 2 : 0))
{
	void* pBuffer = getData().getData();
	int o = 0;

	CEQPacket::FillWord(pBuffer, o++ * 2, header | EQ_MSG_SEQ);
	CEQPacket::FillWord(pBuffer, o++ * 2, packetID);

	if(header & EQ_MSG_ACK)
		CEQPacket::FillWord(pBuffer, o++ * 2, ackID);

	CEQPacket::FillWord(pBuffer, o++ * 2, sequenceID);

	if(header & EQ_MSG_MSG)
	{
		((unsigned char*)pBuffer)[o * 2]		= 0x01;
		((unsigned char*)pBuffer)[o++ * 2 + 1]	= messageID;
	}

	init(o * 2,	pData, dataSize);
}*/

// The only rule for this constructor is that there must be a valid SEQ and packet id.
CEQTxPacket::CEQTxPacket(
		unsigned short header,
		unsigned short packetID,
		unsigned short ackID,
		unsigned short sequenceID,
		unsigned short fragID,
		unsigned short fragIndex,
		unsigned short maxFragIndex,
		unsigned char messageID,
		const void* pData,
		size_t dataSize)
	: CDatagramPacket(
				700,
				dataSize + 16 + 
				(header & EQ_MSG_ACK ? 2 : 0) + 
				(header & EQ_MSG_FRG ? 6 : 0) + 
				(header & EQ_MSG_MSG ? 2 : 0))
{
	void* pBuffer = getData().getData();
	int o = 0;

	CEQPacket::FillWord(pBuffer, o++ * 2, header | EQ_MSG_SEQ);
	CEQPacket::FillWord(pBuffer, o++ * 2, packetID);

	if(header & EQ_MSG_ACK)
		CEQPacket::FillWord(pBuffer, o++ * 2, ackID);

	CEQPacket::FillWord(pBuffer, o++ * 2, sequenceID);

	if(header & EQ_MSG_FRG)
	{
		CEQPacket::FillWord(pBuffer, o++ * 2, fragID);
		CEQPacket::FillWord(pBuffer, o++ * 2, fragIndex);
		CEQPacket::FillWord(pBuffer, o++ * 2, maxFragIndex);
	}

	if(header & EQ_MSG_MSG)
	{
		((unsigned char*)pBuffer)[o * 2]		= 0x01;
		((unsigned char*)pBuffer)[o++ * 2 + 1]	= messageID;
	}

	init(o * 2, pData, dataSize);
}

void CEQTxPacket::init(
		size_t headerSize,
		const void* pData,
		size_t dataSize)
{
	// set the data size apropriately
	getData().setDataSize(dataSize + headerSize + 4);
	void* pBuffer = getData().getData();

	// set data (if any)
	if(dataSize)
		memcpy((unsigned char*)pBuffer + headerSize, pData, dataSize);

	// create and set checksum
	CEQPacket::FillDWord(pBuffer, dataSize + headerSize,
		CUDPServerSocket::getChk().getChecksum(pBuffer, dataSize + headerSize, 0));
}

